/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */
import { SMouseEvent } from "@persagy-web/base";
import { SEquipItem } from "@persagy-web/big";
import { SGraphItem, STextItem, SGraphCircleItem } from "@persagy-web/graph/lib";
import { SColor, SFont, SPoint, SPainter } from "@persagy-web/draw/lib";
import { SCircleItem } from "./SCircleItem";
// 样式接口
interface Style {
    // 字体颜色
    color: string;
    // 字体大小
    fontSize: number;
    // 公式(信息点)
    formula: any[];
    // 图标地址
    url: string;
}
/**
 * 平面图派生设备类
 *
 *  @author  贠星 <yunxing0102@163.com>
 */

export class EquipItem extends SEquipItem {
    StatusPoint: SCircleItem | null = null;

    // 颜色
    private _color: SColor = new SColor();
    get color() {
        return this._color;
    }
    set color(val) {
        this._color = val;
    }

    // 字体
    private _font: SFont = new SFont("sans-serif", 12);
    get font() {
        return this._font;
    }
    set font(val) {
        this._font = val;
    }

    // 设置设备类样式
    setStyle(style: Style) {
        const { color, fontSize, formula, url } = style;
        this.url = url;
        // 更改颜色,字体
        this.color = new SColor(color);
        this.font = new SFont("sans-serif", fontSize);
        this.formula = formula;
        this.update();
    }

    /** 公式(信息点) */
    private _formula: any[] = [];
    get formula(): any[] {
        return this._formula;
    }
    set formula(val) {
        this._formula = val;
        try {
            // const textList = JSON.parse(this._formula);
            if (this.scene) {
                this.textItemList.forEach((textItem) => {
                    this.scene?.removeItem(textItem);
                });
            }
            if (this._formula.length) {
                // if (textList.length) {
                const textItemList: any[] = [];
                this._formula.forEach((item: any, index: number) => {
                    // textList.forEach((item: any, index: number) => {
                    const obj = new STextItem(this);
                    // @ts-ignore
                    obj.propertyData = { ...item };
                    obj.text = item.name;
                    if (item.pos) {
                        obj.moveTo(item.pos.x, item.pos.y);
                    } else {
                        obj.moveTo(this.img.width * 0.5, this.font.size * (index - 0.125 - 0.5 * this._formula.length));
                    }
                    // TODO: obj.moveTo(this.img.width * 0.5, -(this.font.size * 1.25 * 0.5) + (index * this.font.size) - (textList.length - 1) * 0.5 * this.font.size);
                    // obj.moveTo(this.img.width * 0.5, this.font.size * (index - 0.125 - 0.5 * textList.length));
                    // obj.connect("onMove", this, this.textMove.bind(this));
                    // console.log("::::", this.font, this.color);
                    obj.font = this.font;
                    obj.color = this.color;
                    obj.isTransform = false;
                    obj.showSelect = false;
                    // console.log("::::::", obj);
                    textItemList.push(obj);
                });
                this.textItemList = textItemList;
            } else {
                this.textItemList = [];
            }
            this.update();
        } catch (error) {
            console.error("公式数据错误", error);
        }
    }

    constructor(parent: SGraphItem | null, data: Style) {
        super(parent);
        this.zOrder = 9700;
        this.isTransform = false;
        this.sWidth = 32;
        this.sHeight = 32;
        this.img.showSelect = false;
        this.setStyle(data);
    }
    // 设置设备名称
    setEquipName() {
        const item = new STextItem(this);
        item.text = this.data.localName;
        // item.strokeColor = new SColor('#6b7086');
        item.color = new SColor("#6b7086");
        item.font = new SFont("sans-serif", 12);
        item.isTransform = true;
        // item.font = new SFont("sans-serif", 16);
        // item.moveTo(-this.width / 2, this.height / 2);
        item.moveTo(-this.width / 2, 20);
        this.setStatusPoint(item);
    }

    /**
     * 设置状态点
     *
     * @param parent 父类
     */
    setStatusPoint(parent: STextItem | null) {
        const item = new SCircleItem(parent);
        const h = parent ? parent.height : 0;
        item.localtion = new SPoint(0, 0);
        item.radius = 4;
        item.fillColor = new SColor("#de6466");
        item.strokeColor = new SColor("#de6466");
        item.moveTo(-item.radius * 2, h);
        this.StatusPoint = item;
    }

    /**
     * 设置状态远点颜色
     *
     * @param val 颜色字符
     */
    setStatusPointColor(val: string) {
        if (!this.StatusPoint) return;
        this.StatusPoint.fillColor = new SColor(val);
        this.StatusPoint.strokeColor = new SColor(val);
    }

    /**
     * 获取信息点数组
     */
    getMsgList(): any {
        return this.textItemList;
    }

    onMouseMove(event: SMouseEvent): boolean {
        const scene = this.scene;
        if (null != scene) {
            if (scene.hoverItem == null || scene.hoverItem !== this) {
                if (scene.hoverItem != null) {
                    scene.hoverItem.onMouseLeave(event);
                }
                this.onMouseEnter(event);
                scene.hoverItem = this;
            }
        }
        return true;
    }
    /**
     * 鼠标进入事件
     *''
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseEnter(event: SMouseEvent): boolean {
        this.$emit("onMouseEnter");
        return false;
    }

    /**
     * 鼠标离开事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseLeave(event: SMouseEvent): boolean {
        this.$emit("onMouseLeave");
        return false;
    }
    // 显示图片阴影
    _showImgShadow = false;
    get showImgShadow(): boolean {
        return this._showImgShadow;
    }
    set showImgShadow(v: boolean) {
        this._showImgShadow = v;
        this.update();
    }
    onDraw(painter: SPainter): void {
        if (this.showImgShadow) {
            painter.shadow.shadowBlur = 20;
            painter.shadow.shadowColor = new SColor(`#000000`);
            painter.shadow.shadowOffsetX = 10;
            painter.shadow.shadowOffsetY = 10;
        } else {
            painter.shadow.shadowColor = SColor.Transparent;
        }
    }
}
