/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */
import { SGraphItem, STextItem, SLineStyle, STextOrigin, Point } from "@persagy-web/graph";
import { SColor, SFont, SPainter, SLineCapStyle, SPoint, SPath, SPolygonUtil, SRect } from "@persagy-web/draw";
import { SMouseEvent } from "@persagy-web/base";
// 样式接口
interface Style {
    // 轮廓线
    outline?: any[];
    // 字体颜色
    color: string;
    // 填充颜色
    fillColor: string;
    // 边框颜色
    strokeColor: string;
    // 字体大小
    fontSize: number;
    // 公式(信息点)
    formula: any[];
    // 线型
    lineStyle: number;
    // 线宽
    lineWidth: number;
}

/**
 * 平面图派生空间类
 *
 *  @author  贠星 <yunxing0102@163.com>
 */
export class ZoneItem extends SGraphItem {
    /** X 坐标最小值 */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X 坐标最大值 */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y 坐标最小值 */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y 坐标最大值 */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 墙轮廓线坐标 list */
    private pointList: SPoint[][][] = [];
    /** path 对象 */
    private pathList: SPath[] = [];
    /** 文本数组 */
    textItemList: STextItem[] = [];

    /** 线型 */
    _lineStyle: number = SLineStyle.Solid;
    get lineStyle() {
        return this._lineStyle;
    }
    set lineStyle(val) {
        this._lineStyle = val;
        this.update();
    }

    /** 线宽 */
    _lineWidth = 1;
    get lineWidth() {
        return this._lineWidth;
    }
    set lineWidth(val) {
        this._lineWidth = val;
        this.update();
    }

    /** 颜色 */
    _color: SColor = SColor.Black;
    get color(): SColor {
        return this._color;
    }
    set color(v: SColor) {
        this._color = new SColor(v);
        this.update();
    }
    /** 边框颜色 */
    _strokeColor: SColor = SColor.Black;
    get strokeColor(): SColor {
        return this._strokeColor;
    }
    set strokeColor(v: SColor) {
        this._strokeColor = new SColor(v);
        this.update();
    }

    /** 填充颜色 */
    _fillColor: SColor = SColor.Black;
    get fillColor(): SColor {
        return this._fillColor;
    }
    set fillColor(v: SColor) {
        this._fillColor = new SColor(v);
        this.update();
    }
    // 字体
    private _font: SFont = new SFont("sans-serif", 12);
    get font() {
        return this._font;
    }
    set font(val) {
        this._font = val;
    }
    /** 是否显示文字 */
    _showText = true;
    get showText(): boolean {
        return this._showText;
    }
    set showText(v: boolean) {
        if (v === this._showText) {
            return;
        }
        this._showText = v;
        if (v) {
            this.textItemList.map((item) => {
                item.show();
            });
        } else {
            this.textItemList.map((item) => {
                item.hide();
            });
        }
    }
    /** 公式(信息点) */
    private _formula: any[] = [];
    get formula(): any[] {
        return this._formula;
    }
    set formula(val) {
        this._formula = val;
        try {
            if (this.scene) {
                this.textItemList.forEach((textItem) => {
                    this.scene?.removeItem(textItem);
                });
            }
            if (val.length) {
                const textItemList: any[] = [];
                val.forEach((item: any, index: number) => {
                    const obj = new STextItem(this);
                    obj.propertyData = { ...item };
                    obj.text = item.name;
                    if (item.pos) {
                        obj.moveTo(item.pos.x, item.pos.y);
                    } else {
                        const avgPos = this.getAverageVal();
                        const scale = this.scene?.view?.scale;
                        if (scale) {
                            obj.moveTo(avgPos.avgX, avgPos.avgY - (this.font.size * (0.5 * val.length - 0.5 - index)) / scale);
                        } else {
                            obj.moveTo(avgPos.avgX, avgPos.avgY - this.font.size * (0.5 * val.length - 0.5 - index));
                        }
                    }

                    obj.connect("onMove", this, this.textMove.bind(this));
                    obj.font = this.font;
                    obj.color = this.color;
                    obj.originPosition = STextOrigin.Center;
                    obj.isTransform = false;
                    obj.showSelect = false;
                    textItemList.push(obj);
                });
                this.textItemList = textItemList;
            } else {
                this.textItemList = [];
            }
            this.update();
        } catch (error) {
            console.error("公式数据错误", error);
        }
    }

    constructor(parent: SGraphItem | null, style: Style) {
        super(parent);
        this.zOrder = 9500;
        this.setStyle(style);
    }
    /** 设置空间样式 */
    setStyle(style: Style) {
        const { outline, color, fillColor, strokeColor, fontSize, formula, lineStyle, lineWidth } = style;
        this.color = new SColor(color);
        this.fillColor = new SColor(fillColor);
        this.strokeColor = new SColor(strokeColor);
        this.lineStyle = lineStyle;
        this.lineWidth = lineWidth;

        this.font = new SFont("sans-serif", fontSize);
        this.setOutline(outline);
        this.formula = formula;
    }
    /**设置轮廓线 */
    setOutline(outline) {
        if (outline) {
            const _outline = outline.map((t): Point[][] => {
                return t.map((it): Point[] => {
                    return it.map(
                        (item): Point => {
                            return { x: item.X, y: -item.Y };
                        }
                    );
                });
            });
            this.minX = outline[0][0][0].x;
            this.maxX = this.minX;
            this.minY = outline[0][0][0].y;
            this.maxY = this.minY;
            // 处理轮廓点数组,同时计算最大最小值
            this.pointList = _outline.map((t): SPoint[][] => {
                const sPath = new SPath();
                const tempArr = t.map((it): SPoint[] => {
                    const array = it.map(
                        (item): SPoint => {
                            const x = item.x,
                                y = item.y;
                            if (x < this.minX) {
                                this.minX = x;
                            }
                            if (y < this.minY) {
                                this.minY = y;
                            }
                            if (x > this.maxX) {
                                this.maxX = x;
                            }
                            if (y > this.maxY) {
                                this.maxY = y;
                            }
                            return new SPoint(x, y);
                        }
                    );
                    sPath.polygon(array);
                    return array;
                });
                this.pathList.push(sPath);
                return tempArr;
            });
        } else {
            this.minX = 0;
            this.maxX = 0;
            this.minY = 0;
            this.maxY = 0;
        }
    }
    /** 设置缩放比例,缩放后,调整信息点的位置 */
    setScale(scale: number) {
        this.textItemList.forEach((item, index) => {
            if (item.propertyData.pos) {
                item.moveTo(item.propertyData.pos.x, item.propertyData.pos.y);
            } else {
                const avgPos = this.getAverageVal();
                item.moveTo(avgPos.avgX, avgPos.avgY - (this.font.size * (0.5 * this.textItemList.length - 0.5 - index)) / scale);
            }
        });
    }

    /**
     * 获取轮廓线第一块区域的坐标平均值
     */
    getAverageVal() {
        let avgX = 0,
            avgY = 0;
        if (this.pointList[0] && this.pointList[0][0] && this.pointList[0][0].length) {
            const firstOutline = this.pointList[0][0];
            avgX =
                firstOutline.reduce((val, pos) => {
                    return val + pos.x;
                }, 0) / firstOutline.length;
            avgY =
                firstOutline.reduce((val, pos) => {
                    return val + pos.y;
                }, 0) / firstOutline.length;
        }
        return { avgX, avgY };
    }

    /**
     * 文本位置移动回调函数
     *
     * @param textItem 移动的文本元素
     * @param pointList 移动前后位置坐标信息
     */
    textMove(textItem: STextItem, pointList: SPoint[]) {
        this.moveText();
    }
    moveText() {}
    onMouseDown(event: SMouseEvent): boolean {
        this.$emit("onMouseDown");
        return false;
    }
    onMouseMove(event: SMouseEvent): boolean {
        const scene = this.scene;
        if (null != scene) {
            if (scene.hoverItem == null || scene.hoverItem !== this) {
                if (scene.hoverItem != null) {
                    scene.hoverItem.onMouseLeave(event);
                }
                this.onMouseEnter(event);
                scene.hoverItem = this;
            }
        }
        return true;
    }
    /**
     * 鼠标进入事件
     *''
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseEnter(event: SMouseEvent): boolean {
        this.$emit("onMouseEnter");
        return false;
    }

    /**
     * 鼠标离开事件
     *
     * @param event   保存事件参数
     * @return 是否处理事件
     */
    onMouseLeave(event: SMouseEvent): boolean {
        this.$emit("onMouseLeave");
        return false;
    }
    /**
     * Item 对象边界区域
     *
     * @return 对象边界区域
     */
    boundingRect(): SRect {
        let rect = new SRect(this.minX, this.minY, this.maxX - this.minX, this.maxY - this.minY);
        // if (this.showText) {
        //     this.textItemList.forEach((item) => {
        //         rect = rect.unioned(item.boundingRect().adjusted(item.pos.x, item.pos.y, 0, 0));
        //     });
        // }
        return rect.adjusted(-5, -5, 10, 10);
    }
    /**
     * 判断点是否在区域内
     *
     * @param x 点 x 坐标
     * @param y 点 y 坐标
     * @return 是否在区域内
     */
    contains(x: number, y: number): boolean {
        for (let j = 0; j < this.pointList.length; j++) {
            const arr = this.pointList[j];
            if (arr.length < 1 || !SPolygonUtil.pointIn(x, y, arr[0])) {
                continue;
            }
            if (arr.length == 1) {
                return true;
            }
            let flag = false;
            for (let i = 1; i < arr.length; i++) {
                if (SPolygonUtil.pointIn(x, y, arr[i])) {
                    flag = true;
                    break;
                }
            }
            if (flag) {
                continue;
            }
            return true;
        }
        return false;
    }
    onDraw(painter: SPainter): void {
        const rect = this.boundingRect();

        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.strokeColor;
        painter.brush.color = this.fillColor;
        painter.pen.lineWidth = painter.toPx(this.lineWidth);

        if (this.lineStyle == SLineStyle.Dashed) {
            painter.pen.lineDash = [painter.toPx(this.lineWidth * 3), painter.toPx(this.lineWidth * 7)];
        } else if (this.lineStyle == SLineStyle.Dotted) {
            painter.pen.lineDash = [painter.toPx(this.lineWidth), painter.toPx(this.lineWidth)];
        }
        // 是否选中
        if (this.selected) {
            painter.shadow.shadowBlur = 10;
            painter.shadow.shadowColor = new SColor(`#00000033`);
            painter.shadow.shadowOffsetX = 5;
            painter.shadow.shadowOffsetY = 5;
        } else {
            painter.shadow.shadowColor = SColor.Transparent;
        }

        this.pathList.forEach((t): void => {
            painter.drawPath(t);
        });
    }
}
